TITLE  A-Current

COMMENT
ENDCOMMENT

UNITS
  {
  (mA) = (milliamp)
  (mV) = (millivolt)
  (molar) = (1/liter)
  (mM) = (millimolar)
  (S) = (mho)
  (uS) = (microS)
  }
 
NEURON  : this block manages the interface with NEURON
  {
  SUFFIX a
  :USEION k READ ek WRITE ik
  NONSPECIFIC_CURRENT i
  RANGE gbar, g, minf, hinf, taum, tauh, vslopem, vslopeh, vhalfm, vhalfh,e, i
  }


PARAMETER 
  {
  gbar = 0 (uS/mm2)

  taum = 15 (ms)
  tauh = 100 (ms)
  }

ASSIGNED  
  {
  v (mV)
  :ek (mV)  
  e (mV)
  minf
  hinf
  : taum (ms)
  : tauh (ms)
  g (uS/mm2)
  i (mA/cm2)
  vhalfm (mV)
  vslopem (mV)
  vhalfh (mV)
  vslopeh (mV)
  }

STATE
  {
  m
  h
  }
 
BREAKPOINT
  {
  SOLVE state_change METHOD cnexp
  g = (m^2)*(h)*gbar
  i = (1e-4)*(g*(v-e))
  }

DERIVATIVE state_change
  {
  rates(v)
  m' = (minf-m)/taum
  h' = (hinf-h)/tauh
  }

INITIAL 
  {
  rates(v)
  m = minf
  h = hinf
  }

PROCEDURE rates(v(mV)) 
  {  

  minf = 1 / (1 + exp( (v - vhalfm ) / vslopem) )
  hinf = 1 / (1 + exp( (v - vhalfh ) / vslopeh) )

  
  }
