TITLE Cancer LP Ca current, drives KCa

COMMENT
ENDCOMMENT

UNITS
  {
  (pure) = (1)
  (mA) = (milliamp)
  (mV) = (millivolt)
  (mol) = (1)
  (nmol) = (nanomol)
  (molar) = (mol/liter)
  (mM) = (millimolar)
  (uM) = (micromolar)
  (uC) = (microC)
  (S) = (mho)
  (uS) = (microS)
  (pmol) = (picomole)
  (pC) = (picocoulomb)
  (J) = (joule)
  (um) = (micrometer)
  k = 1.381e-23 (J/K)
  q_e = 1.602e-19 (coulomb) 
  N_A = 6.022e23 (pure)
  z_ca = 2 (pure)
  }

NEURON
  {
  SUFFIX ca
  USEION ca READ cai, cao WRITE ica
  RANGE minf, hinf, taum, tauh, vhalfm, vslopem, frac, ghknow, alphaTaum, betaTaum, vhalftaum, vslopetaum, hinf_cai, alphaTauh, betaTauh, vhalftauh, vslopetauh, Pbar, v, ica, cao, cai
  : 
  }


PARAMETER  
  {
  Pbar = 0 (nm/ms)
   
  }

ASSIGNED
  {
  : things assigned to outside of this code
  v (mV)
  cao (mM)
  cai (mM)
  celsius (degC)
  minf
  hinf
  taum (ms)
  tauh (ms)
  vhalfm (mV)
  vslopem (mV)
  frac
  ghknow (uC/cm2/nm)
  ica (mA/cm2)
  alphaTaum (ms)
  betaTaum (ms)
  vhalftaum (mV)
  vslopetaum (mV)
  alphaTauh (ms)
  betaTauh (ms)
  vhalftauh (mV)
  vslopetauh (mV)  
  hinf_cai (mM)

  }

STATE  : also a var-decl block, where one declares state vars local to
       : this particular mechanism
  {
  m
  h
  }

BREAKPOINT
  {
  SOLVE state_change METHOD cnexp
  ghknow=ghk(v,cai,cao,z_ca)
  frac = m^3*h
  ica = Pbar*frac*ghknow
  }

DERIVATIVE state_change
  {
  rates(v,cai)
  m' = (minf-m)/taum
  h' = (hinf-h)/tauh
  }

INITIAL
  {
  rates(v,cai)
  m = minf
  h = hinf
  }


PROCEDURE rates(v(mV),cai(mM) ) 
  { 
  
  minf= 1 / (1 + exp( (v - vhalfm) / vslopem ) )   
  hinf = 1/(1+(cai/hinf_cai ))

  taum = alphaTaum + betaTaum / (1 + exp( (v - vhalftaum) / vslopetaum ))
  tauh = alphaTauh + betaTauh / (1 + exp( (v - vhalftauh) / vslopetauh ))
  }

FUNCTION ghk(v(mV), ci(mM), co(mM), z) (uC/cm2/nm) 
  {
  LOCAL eta, eci, eco
  eta = (1e-3)*z*q_e*v/(k*(celsius+273.15))
  eco = co*efun(eta)
  eci = ci*efun(-eta)
  ghk = (1e-7)*(N_A*q_e*z_ca*(eci - eco))
  }

FUNCTION efun(eta)
  {
  if (fabs(eta) < 1e-4)
    {
    efun = 1 - eta/2
    }
  else
    {
    efun = eta/(exp(eta) - 1)
    }
  }
