COMMENT
This file, h.mod, implements the Ih hyperpolarization activated cation nonspecific
current from Liu et al. 1998 (Activity dependent conductances) table p.2319
Tom M Morse 20070803
ENDCOMMENT

NEURON {
	SUFFIX hliu
	NONSPECIFIC_CURRENT i
    RANGE gbar, g, vslopem, vhalfm, i, e, taum_vhalf, taum_vslope, taum_alpha, taum_beta, taum, minf
}

UNITS {
  (mA) = (milliamp)
  (mV) = (millivolt)
  (molar) = (1/liter)
  (mM) = (millimolar)
  (S) = (mho)
  (uS) = (microS)
}

PARAMETER {
	gbar = 0 (uS/mm2)
	e = -20 (mV)
    taum_vhalf = -42.2 (mV)
    taum_vslope = 8.73 (mV)
    taum_alpha = 272 (ms)
    taum_beta = 1499 (ms)
    vhalfm = -70 (mV)
    vslopem = 6 (mV)
}

ASSIGNED {
	i (mA/cm2)
	v (mV)
	g (uS/mm2)
	minf
    taum (ms)

}

STATE { m }

BREAKPOINT {
	SOLVE states METHOD cnexp
	g = gbar * m
	i = (1e-4)*(g * (v - e))
}

INITIAL {
	rates(v)
	m = minf
}
DERIVATIVE states {
	rates(v)
	m' = (minf - m)/taum
}

FUNCTION tau_m(Vm (mV)) (ms) {
	tau_m = taum_alpha+taum_beta/(1+exp(-(Vm - taum_vhalf)/taum_vslope))
}

PROCEDURE rates(Vm(mV)) {
	taum  = tau_m(Vm)
	minf = 1/(1+exp((Vm-vhalfm)/vslopem))
}
