TITLE  Potassium

COMMENT
ENDCOMMENT

UNITS
  {
  (mA) = (milliamp)
  (mV) = (millivolt)
  (molar) = (1/liter)
  (mM) = (millimolar)
  (S) = (mho)
  (uS) = (microS)
  }
 
NEURON
  {
  SUFFIX k
  NONSPECIFIC_CURRENT i
  RANGE gbar, g, minf, taum, vslopem, vhalfm, i, e, alphaTaum, betaTaum
  }


PARAMETER 
  {
  gbar = 0 (uS/mm2)
  alphaTaum = 2 (ms)
  betaTaum = 14 (ms)
  }

ASSIGNED  
  {
  v (mV)
  e (mV)
  minf
  g (uS/mm2)
  i (mA/cm2)
  vhalfm (mV)
  vslopem (mV)
  taum (ms)
  }

STATE
  {
  m
  }
 
BREAKPOINT
  {
  SOLVE state_change METHOD cnexp
  g = (m^4)*gbar
  i = (1e-4)*(g*(v-e))
  }

DERIVATIVE state_change
  {
  rates(v)
  m' = (minf-m)/taum
  }

INITIAL 
  {
  rates(v)
  m = minf
  }

PROCEDURE rates(v(mV)) 
  {  

  minf = 1 / (1 + exp( (v - vhalfm ) / vslopem) )
  taum = alphaTaum + betaTaum * (1 - minf)
  }
