TITLE Cancer LP KCa current

COMMENT
ENDCOMMENT

UNITS
  {
  (pure) = (1)
  (mA) = (milliamp)
  (mV) = (millivolt)
  (mol) = (1)
  (nmol) = (nanomol)
  (molar) = (mol/liter)
  (mM) = (millimolar)
  (uM) = (micromolar)
  (uC) = (microC)
  (S) = (mho)
  (uS) = (microS)
  (pmol) = (picomole)
  (pC) = (picocoulomb)
  (J) = (joule)
  (um) = (micrometer)
  }

NEURON 
  {
  SUFFIX kca
  NONSPECIFIC_CURRENT i
  USEION ca READ cao, cai
  RANGE gbar, g,  e, i, caiFactm, caiFactExpm, vhalfm, vslopem, alphaTaum, betaTaum, vhalfTaum, vhalfTaumShift, caiFactTaum, deltaTaum, tauh, caiFacth, caiFactExph, minf, hinf, taum
 
  }


PARAMETER
  {
  gbar = 0      (uS/mm2)
  }

ASSIGNED
  {
  v (mV)
  cai (mM)
  cao (mM)
  celsius (degC)
  e (mV)
  minf 
  taum (ms)
  hinf
  g (uS/mm2)
  i (mA/cm2)
  caiFactm (mM)
  caiFactExpm
  caiFacth (mM)
  caiFactExph
  vslopem (mV)
  vhalfm (mV)
  alphaTaum (ms)
  betaTaum (ms)
  vhalfTaum (mV)
  vhalfTaumShift (mV)
  caiFactTaum(mM)
  deltaTaum (mV)
  tauh
  }

STATE
  {
  m
  h
  }

BREAKPOINT
  {
  SOLVE state_change METHOD cnexp
  g = gbar*m*h
  i = (1e-4)*(g*(v-e))
  }

DERIVATIVE state_change
  {
  rates(v,cai)
  m' = (minf-m)/taum
  h' = (hinf-h)/tauh :11.85(ms)
  }

INITIAL
  {
  rates(v,cai)
  m=minf
  h=hinf
  }

PROCEDURE rates(v(mV),cai(mM)) 
  {  
  if (cai<1e-9)
    {
    cai=1e-9(mM)
    }

  minf = 1/(1+(cai/caiFactm)^(caiFactExpm)) * 1 / ( 1 + exp( (v-vhalfm) / vslopem ) )

  taum = alphaTaum + (betaTaum / (1+exp(-(v+(vhalfTaum-vhalfTaumShift*log(cai/(caiFactTaum))))*deltaTaum )) )

  hinf = 1/(1+(cai/caiFacth)^(caiFactExph))

  }
